(function(Picket, _){
	
	_.Constant = function(definition, isFromInterface, value, typeChecker, accessController)
	{
		if (!(definition instanceof Picket.Member.Constant.Definition)) {
			throw new _.Constant.Fatal(
				'NO_DEFINITION_PROVIDED',
				'Provided type: ' + typeof definition
			);
		}
		if (isFromInterface !== false) {
			throw new _.Constant.Fatal('IS_FROM_INTERFACE');
		}
		if (!(typeChecker instanceof Picket.TypeChecker)) {
			throw new _.Constant.Fatal(
				'NO_TYPE_CHECKER_PROVIDED',
				'Provided type: ' + typeof typeChecker
			);
		}
		if (!(accessController instanceof Picket.Access.Controller)) {
			throw new _.Constant.Fatal(
				'NO_ACCESS_CONTROLLER_PROVIDED',
				'Provided type: ' + typeof accessController
			);
		}
		if (['undefined', 'string', 'number'].indexOf(typeof value) == -1) {
			throw new _.Constant.Fatal(
				'INVALID_VALUE_TYPE',
				'Provided type: ' + typeof value
			);
		}
		if (typeof value != 'undefined') {
			var typeIdentifier = definition.getTypeIdentifier();
			var isValid = typeChecker.isValidType(value, typeIdentifier);
			if (isValid !== true) {
				throw new _.Constant.Fatal(
					'INVALID_VALUE',
					'Constant type: ' + typeIdentifier
				);
			}
			this._isAutoGenerated = false;
		} else {
			value = _generateValue(definition.getTypeIdentifier());
			this._isAutoGenerated = true;
		}
		this._value = value;
		this._definition = definition;
		this._accessController = accessController;
	};
	
	_.Constant.prototype.getName = function()
	{
		return this._definition.getName();
	};
	
	_.Constant.prototype.getAccessTypeIdentifier = function()
	{
		// @todo Method untested
		return this._definition.getAccessTypeIdentifier();
	};
	
	_.Constant.prototype.getTypeIdentifier = function()
	{
		// @todo Method untested
		return this._definition.getTypeIdentifier();
	};
	
	_.Constant.prototype.isAutoGenerated = function()
	{
		// @todo Method untested
		return this._isAutoGenerated;
	};
	
	_.Constant.prototype.get = function(targetConstructor, accessInstance)
	{
		// @todo This if statment is a hack. This
		// class should not control access and it
		// causes an issue for Reflection.Constant.
		// Originally all access checking was done,
		// not just when the arguments are provided.
		// Also, note that the related test is disabled.
		if (targetConstructor && accessInstance) {
			if (typeof targetConstructor != 'function') {
				throw new _.Constant.Fatal(
					'NON_FUNCTION_TARGET_CONSTRUCTOR_PROVIDED',
					'Provided type: ' + typeof targetConstructor
				);
			}
			var canAccess = this._accessController.canAccess(
				targetConstructor,
				accessInstance,
				this._definition.getAccessTypeIdentifier()
			);
			if (canAccess !== true) throw new _.Constant.Fatal('ACCESS_NOT_ALLOWED');
		}
		return this._value;
	};
	
	var _generateValue = function(type)
	{
		if (type == 'string') {
			var string = '';
			var chars = [
				'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
				'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F',
				'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
				'W', 'X', 'Y', 'Z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
			];
			for (var i = 0; i < 32; i++) {
				var randomIndex = Math.floor(Math.random() * 26);
				string = string + chars[randomIndex];
			}
			return string;
		} else if (type == 'number') {
			return Math.floor(Math.random() * 9007199254740992);
		}
	};
	
})(
	window.Picket = window.Picket || {},
	window.Picket.Member = window.Picket.Member || {}
);
