(function(Picket, _){
	
	_.Method = function(definition, isFromInterface, value, typeChecker, accessController)
	{
		if (!(definition instanceof Picket.Member.Method.Definition)) {
			throw new _.Method.Fatal(
				'NO_DEFINITION_PROVIDED',
				'Provided type: ' + typeof definition
			);
		}
		if (!(typeChecker instanceof Picket.TypeChecker)) {
			throw new _.Method.Fatal(
				'NO_TYPE_CHECKER_PROVIDED',
				'Provided type: ' + typeof typeChecker
			);
		}
		if (!(accessController instanceof Picket.Access.Controller)) {
			throw new _.Method.Fatal(
				'NO_ACCESS_CONTROLLER_PROVIDED',
				'Provided type: ' + typeof accessController
			);
		}
		if (isFromInterface === true || definition.isAbstract() === true) {
			if (value !== null && typeof value != 'undefined' && value !== '') {
				throw new _.Method.Fatal('UNEXPECTED_IMPLEMENTATION');
			}
			this._isAbstract = true;
		} else {
			if (typeof value != 'function') {
				throw new _.Method.Fatal(
					'NON_FUNCTION_IMPLEMENTATION',
					'Provided type: ' + typeof value
				);
			}
			if (!definition.hasArgumentTypes()) {
				throw new _.Method.Fatal('NON_ABSTRACT_METHOD_DECLARED_WITH_NO_ARGUMENT_TYPES');
			}
			this._isAbstract = false;
		}
		if (definition.hasArgumentTypes()) {
			var types = definition.getArgumentTypeIdentifiers();
			for (var i = 0; i < types.length; i++) {
				if (types[i] === 'undefined') throw new _.Method.Fatal('UNDEFINED_ARGUMENT_TYPE');
				if (types[i] === 'null') throw new _.Method.Fatal('NULL_ARGUMENT_TYPE');
			}
		}
		this._value = value;
		this._definition = definition;
		this._typeChecker = typeChecker;
		this._accessController = accessController;
	};
	
	_.Method.prototype.getName = function()
	{
		return this._definition.getName();
	};
	
	_.Method.prototype.hasArgumentTypes = function()
	{
		// @todo Untested method
		return (this._definition.hasArgumentTypes()) ? true : false;
	};
	
	_.Method.prototype.getArgumentTypeIdentifiers = function()
	{
		return this._definition.getArgumentTypeIdentifiers();
	};
	
	_.Method.prototype.getReturnType = function()
	{
		// @todo Untested method
		return this._definition.getReturnTypeIdentifier();
	};
	
	_.Method.prototype.isStatic = function()
	{
		return this._definition.isStatic();
	};
	
	_.Method.prototype.isAbstract = function()
	{
		return this._isAbstract;
	};
	
	// @todo This method not tested
	_.Method.prototype.argumentIsOptional = function(index)
	{
		return this._definition.argumentIsOptional(index);
	};
	
	// @todo This method not tested
	_.Method.prototype.getDefaultArgumentValue = function(index)
	{
		return this._definition.getDefaultArgumentValue(index);
	};
	
	_.Method.prototype.getAccessTypeIdentifier = function()
	{
		return this._definition.getAccessTypeIdentifier();
	};
	
	_.Method.prototype.call = function(target, localTarget, accessInstance, args, scopeVariables)
	{
		if (this._isAbstract) throw new _.Method.Fatal('INTERACTION_WITH_ABSTRACT');
		if (typeof target != 'object' && typeof target != 'function') {
			throw new _.Method.Fatal(
				'NON_OBJECT_OR_CONSTRUCTOR_TARGET_PROVIDED',
				'Provided type: ' + typeof target
			);
		}
		if (typeof localTarget != 'object' && typeof localTarget != 'function') {
			throw new _.Method.Fatal(
				'NON_OBJECT_OR_CONSTRUCTOR_LOCAL_TARGET_PROVIDED',
				'Provided type: ' + typeof localTarget
			);
		}
		if (Object.prototype.toString.call(args) != '[object Array]') {
			throw new _.Method.Fatal(
				'NON_ARRAY_ARGUMENTS_PROVIDED',
				'Provided type: ' + typeof args
			);
		}
		if (typeof scopeVariables != 'undefined' && typeof scopeVariables != 'object') {
			throw new _.Method.Fatal(
				'NON_OBJECT_SCOPE_VARIABLES',
				'Provided type: ' + typeof scopeVariables
			);
		}
		var canAccess = this._accessController.canAccess(
			localTarget,
			accessInstance,
			this._definition.getAccessTypeIdentifier()
		);
		if (canAccess !== true) throw new _.Method.Fatal('ACCESS_NOT_ALLOWED');
		var areValidTypes = this._typeChecker.areValidTypes(
			args,
			this.getArgumentTypeIdentifiers()
		);
		if (scopeVariables) {
			var originalScopeVariables = {};
			for (var i in scopeVariables) {
				if (typeof window[i] != 'undefined') {
					originalScopeVariables[i] = window[i];
				}
				window[i] = scopeVariables[i];
			}
		}
		var currentOwner = this._value.$$owner;
		var currentLocalOwner = this._value.$$localOwner;
		this._value.$$owner = target;
		this._value.$$localOwner = localTarget;
		var returnValue = this._value.apply(target, args);
		this._value.$$owner = currentOwner;
		this._value.$$localOwner = currentLocalOwner;
		if (scopeVariables) {
			for (var i in scopeVariables) {
				if (originalScopeVariables[i]) {
					window[i] = originalScopeVariables[i];
				} else {
					delete window[i];
				}
			}
		}
		var isValidType = this._typeChecker.isValidType(
			returnValue,
			this._definition.getReturnTypeIdentifier(),
			target
		);
		if (isValidType !== true) {
			throw new _.Method.Fatal(
				'INVALID_RETURN_VALUE',
				'Returned type: ' + typeof returnValue + '; ' +
				'Expected type: ' + this._definition.getReturnTypeIdentifier()
			);
		}
		return returnValue;
	};
	
})(
	window.Picket = window.Picket || {},
	window.Picket.Member = window.Picket.Member || {}
);
